package me.asofold.bpl.morecommands.command.action;

import me.asofold.bpl.morecommands.MoreCommands;
import me.asofold.bpl.morecommands.command.AbstractCommand;

import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;

/**
 * Set potion effects (remove old, add new).
 * @author asofold
 *
 */
public class SetEffectCommand extends AbstractCommand<MoreCommands> {

    public SetEffectCommand(MoreCommands access) {
        super(access, "seteffect", "morecommands.command.seteffect");
        this.usage = "Zero args to remove. Multiple args with name[@level][xseconds]. Default level is 0, default seconds is 10.";
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String alias, String[] args) {
        if (!demandPlayer(sender)) {
            return true;
        }
        
        Player player = (Player) sender;
        removePotionEffects(player);
        
        for (int i = 1; i < args.length; i++) {
            String input = args[i].toUpperCase();
            PotionEffectType type = null;
            int level = 0;
            int seconds = 10;
            int end = input.length();
            boolean number = false;
            for (int j = input.length() - 1; j >= 0; j--) {
                char c = input.charAt(j);
                if (Character.isDigit(c)) {
                    // Start parsing a number.
                    number = true;
                } else {
                    if (number) {
                        // Finish collecting a number.
                        Integer n;
                        try {
                            n = Integer.parseInt(input.substring(j + 1, end));
                        } catch (NumberFormatException e) {
                            break;
                        }
                        if (c == '@') {
                            // Level.
                            level = n;
                        }
                        else if (c == 'X') {
                            // Seconds.
                            seconds = n;
                        }
                        number = false;
                        end = j;
                    } else {
                        if (j == 0) {
                            // Finished collecting the name.
                            String name = input.substring(j, end).replace('-', '_').replace('.', '_');
                            type = PotionEffectType.getByName(name);
                        } else {
                            // Continue collecting the name.
                        }
                    }
                }
            }
            if (type == null) {
                sender.sendMessage("Can not interpret: " + input);
            } else {
                player.addPotionEffect(new PotionEffect(type, seconds * 20, level));
            }
        }
        
        return true;
    }

    public static void removePotionEffects(Player player) {
        for (final PotionEffect effect : player.getActivePotionEffects()) {
            player.removePotionEffect(effect.getType());
        }
    }

}
