package me.asofold.bpl.morecommands.command.editworld;

import org.bukkit.Material;
import org.bukkit.block.Block;

public class EditUtil {

    /**
     * Maybe we allow the WorldEdit notation for % and other later.
     * @author asofold
     *
     */
    public static interface BlockModifier {
        public void apply(Block block);
    }

    public static interface BlockMatcher {
        public boolean matches(Block block);
    }

    /**
     * (Legacy) Block specification. For easy setting and replacing.
     * @author asofold
     *
     */
    public static class BlockSpec implements BlockModifier, BlockMatcher {

        public final Material material;
        /** May be null. */
        public final Integer data;
        public final byte dataByte;

        public BlockSpec(Material material, Integer data) {
            this.material = material == null ? Material.AIR : material;
            this.data = data;
            if (data != null) {
                dataByte = data.byteValue(); 
            } else {
                dataByte = 0;
            }
        }

        @Override
        @SuppressWarnings("deprecation")
        public void apply(Block block) {
            block.setType(material);
            if (data != null) {
                block.setData(dataByte);
            }
        }

        @SuppressWarnings("deprecation")
        @Override
        public boolean matches(Block block) {
            if (block == null) {
                // Do not match air here.
                return false;
            }
            return block.getType() == material && (data == null || block.getData() == dataByte);
        }

        /**
         * Parse Material:data or Material, allows block ids.
         * @param input
         * @return null on failure.
         */
        public static BlockSpec matchBlockSpec(String input) {
            if (input == null) {
                return null;
            }
            input = input.trim();
            String matSpec, dataSpec;
            if (input.indexOf(':') != -1) {
                String[] split = input.split(":", 2);
                matSpec = split[0].trim();
                dataSpec = split[1].trim();
            } else {
                matSpec = input;
                dataSpec = null;
            }
            Integer data = null;
            if (dataSpec != null && !dataSpec.isEmpty()) {
                try {
                    data = Integer.parseInt(dataSpec);
                } catch (NumberFormatException ex) {
                    return null;
                }
            }
            matSpec = matSpec.replace('-', '_').replace('.', '_').toUpperCase();
            Material material = Material.matchMaterial(matSpec);
            if (material == null || !material.isBlock()) {
                material = null;
                if (matSpec.equals("SNOW")) {
                    material = Material.SNOW_BLOCK;
                } else if (matSpec.equals("STAIRS")) {
                    material = Material.SMOOTH_STAIRS;
                } else {
                    return null;
                }
            }
            return new BlockSpec(material, data);
        }
    }

    /**
     * Set blocks within a xz-radius and and min/max y-margins (cuboid).
     * 
     * @param modSet
     * @param refBlock
     * @param xzMargin
     * @param dyMin
     * @param dyMax
     */
    public static void setRadius(final BlockModifier modSet, final Block set, final int xzMargin, final int dyMin, final int dyMax) {
        for (int y = dyMin; y <= dyMax; y++) {
            for (int x = -xzMargin; x <= xzMargin; x++) {
                for (int z = -xzMargin; z <= xzMargin; z++) {
                    modSet.apply(set.getRelative(x, y, z));
                }
            }
        }
    }

    public static void replaceRadius(final BlockMatcher match, final BlockModifier set, final Block refBlock, final int xzMargin, final int dyMin, final int dyMax) {
        for (int y = dyMin; y <= dyMax; y++) {
            for (int x = -xzMargin; x <= xzMargin; x++) {
                for (int z = -xzMargin; z <= xzMargin; z++) {
                    final Block block = refBlock.getRelative(x, y, z);
                    if (match.matches(block)) {
                        set.apply(block);
                    }
                }
            }
        }
    }

}
