package me.asofold.bpl.morecommands.command.editworld;

import me.asofold.bpl.morecommands.MoreCommands;
import me.asofold.bpl.morecommands.command.AbstractCommand;
import me.asofold.bpl.morecommands.command.editworld.EditUtil.BlockModifier;
import me.asofold.bpl.morecommands.command.editworld.EditUtil.BlockSpec;

import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.BlockState;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.material.Stairs;
import org.bukkit.util.Vector;

public class PyramidCommand extends AbstractCommand<MoreCommands> {

    public PyramidCommand(MoreCommands access) {
        super(access, "pyramid", "morecommands.command.pyramid");
        this.usage = "Specify material[:data], the pyramid height(pos/neg for direction) [and level width multiplier], beware of snow taking 8 blocks per height, steps 2.";
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String alias, String[] args) {
        if (!demandPlayer(sender)) {
            return true;
        }
        if (args.length < 3 || args.length > 4) {
            return false;
        }
        // Top layer material.
        BlockSpec mat = BlockSpec.matchBlockSpec(args[1]);
        if (mat == null) {
            sender.sendMessage("Can not interpret/use: " + args[1]);
            return false;
        }

        // Pyramid height (teleport to top block for positive, else build downwards below foot position).
        Location loc = ((Player) sender).getLocation();
        Block centerTop = loc.getBlock();
        Integer height = null;
        try {
            height = Integer.parseInt(args[2]);
            if (height < -256 || height > 256) {
                height = null;
            }
        } catch (NumberFormatException ex) {}
        if (height == null) {
            sender.sendMessage("Unsupported height: " + height);
            return false;
        }
        if (height > 0) {
            centerTop = centerTop.getRelative(0, height, 0);
        }
        if (centerTop.getY() - height < 0 || centerTop.getY() > centerTop.getWorld().getMaxHeight()) {
            sender.sendMessage("Pyramid outside of world bounds.");
            return false;
        }
        

        // Width multiplier.
        Integer multiplier = null;
        if (args.length < 4) {
            multiplier = 1;
        } else {
            try {
                multiplier = Integer.parseInt(args[3]);
                if (multiplier <= 0 || multiplier > 12) {
                    multiplier = null;
                }
            } catch (NumberFormatException ex) {}
        }
        if (multiplier == null) {
            sender.sendMessage("Unsupported multiplier: " + args[3]);
            return false;
        }
        // Teleport to top and start building.
        Location tpLoc = centerTop.getLocation().add(new Vector(0.5, 0.0, 0.5));
        tpLoc.setPitch(loc.getPitch());
        tpLoc.setYaw(loc.getYaw());
        ((Player) sender).teleport(tpLoc);
        buildPyramid(centerTop, centerTop.getY() - Math.abs(height.intValue()), mat, multiplier.intValue());
        return true;
    }

    /**
     * The top block stays empty.
     * @param centerTop
     * @param centerBottom
     * @param material
     */
    private void buildPyramid(Block centerTop, int yBottom, BlockSpec mod, int levelWidthMulitplier) {
        BlockModifier baseMod = new BlockSpec(Material.BEDROCK, null);
        // TODO: snow layers, half slabs, stairs.
        if (mod.material.name().toLowerCase().endsWith("step")) {
            // 2 blocks are needed for one level.
            placePyramidBase(centerTop, yBottom, baseMod, 2 * levelWidthMulitplier);
            placePyramidLayerStep(centerTop, yBottom, mod, levelWidthMulitplier);
        }
        else if (mod.material.name().toLowerCase().endsWith("stairs")) {
            // Can not multiply width of level
            placePyramidBase(centerTop, yBottom, baseMod, levelWidthMulitplier);
            placePyramidLayerStairs(centerTop, yBottom, mod.material, levelWidthMulitplier);
        }
        else if (mod.material == Material.SNOW) {
            // 8 blocks are needed per level (No extra block on the most outer edges).
            placePyramidBase(centerTop, yBottom, baseMod, 8 * levelWidthMulitplier);
            placePyramidLayerSnow(centerTop, yBottom, mod.material, levelWidthMulitplier);
        }
        else if (mod.material == Material.LAVA || mod.material == Material.STATIONARY_LAVA) {
            placePyramidBase(centerTop, yBottom, baseMod, levelWidthMulitplier);
            placePyramidLayer(centerTop, yBottom, new BlockSpec(Material.LAVA, null), levelWidthMulitplier);
        }
        else if (mod.material == Material.WATER || mod.material == Material.STATIONARY_WATER) {
            placePyramidBase(centerTop, yBottom, baseMod, levelWidthMulitplier);
            placePyramidLayer(centerTop, yBottom, new BlockSpec(Material.WATER, null), levelWidthMulitplier);
        }
        else {
            // Simply place that block type as if an ordinary full block.
            placePyramidBase(centerTop, yBottom, baseMod, levelWidthMulitplier);
            placePyramidLayer(centerTop, yBottom, mod, levelWidthMulitplier);
        }
    }

    /**
     * The top block stays empty. A player can stand on the top block.
     * @param centerTop
     * @param centerBottom
     * @param bedrock
     */
    private void placePyramidBase(final Block centerTop, final int yBottom, final BlockModifier mod, final int levelWidth) {
        final World world = centerTop.getWorld();
        int width = 0;
        for (int y = centerTop.getY() - 1; y >= yBottom; y --) {
            for (int x = centerTop.getX() - width; x <= centerTop.getX() + width; x++) {
                for (int z = centerTop.getZ() - width; z <= centerTop.getZ() + width; z ++) {
                    mod.apply(world.getBlockAt(x, y, z));
                }
            }
            width += levelWidth;
        }
    }

    /**
     * The layer on top of the fully solid pyramid base. The top block stays
     * empty. A player can stand on the top block.
     * 
     * @param centerTop
     * @param centerBottom
     * @param bedrock
     */
    private void placePyramidLayer(final Block centerTop, final int yBottom, final BlockModifier mod, final int levelWidth) {
        // TODO: This is a somewhat inefficient implementation.
        final int x0 = centerTop.getX();
        final int y0 = centerTop.getY() - 1;
        final int z0 = centerTop.getZ();
        final World world = centerTop.getWorld();
        int width = levelWidth;
        for (int y = y0; y >= yBottom; y --) { // Is absolute.
            final int inner = width - levelWidth; // Coordinate to exempt inside of. 
            for (int x = -width; x <= width; x++) { // Relative.
                for (int z = -width; z <= width; z ++) { // Relative.
                    if (Math.abs(x) > inner || Math.abs(z) > inner) {
                        mod.apply(world.getBlockAt(x + x0, y, z + z0));
                    }
                }
            }
            width += levelWidth;
        }
    }

    private void placePyramidLayerStairs(final Block centerTop, final int yBottom, final Material material, final int levelWidth) {
        // TODO: This is a somewhat inefficient implementation.
        final int x0 = centerTop.getX();
        final int y0 = centerTop.getY() - 1;
        final int z0 = centerTop.getZ();
        final World world = centerTop.getWorld();
        int width = levelWidth;
        for (int y = y0; y >= yBottom; y --) { // Is absolute.
            final int inner = width - levelWidth; // Coordinate to exempt inside of. 
            for (int x = -width; x <= width; x++) { // Relative.
                for (int z = -width; z <= width; z ++) { // Relative.
                    final int xAbs = Math.abs(x);
                    final int zAbs = Math.abs(z);
                    if (xAbs == inner + 1 && zAbs <= inner + 1 || zAbs == inner + 1 && xAbs <= inner + 1) {
                        final Block block = world.getBlockAt(x + x0, y, z + z0);
                        block.setType(material);
                        final BlockFace face;
                        if (xAbs == zAbs) {
                            // Diagonal (!).
                            if (x > 0) {
                                face = BlockFace.WEST;
                                //face = z > 0 ? BlockFace.NORTH_WEST : BlockFace.SOUTH_WEST;
                            } else {
                                face = BlockFace.EAST;
                                //face = z > 0 ? BlockFace.NORTH_EAST : BlockFace.SOUTH_EAST;
                            }
                        } else if (xAbs > zAbs) {
                            face = x > 0 ? BlockFace.WEST : BlockFace.EAST;
                        } else {
                            face = z > 0 ? BlockFace.NORTH : BlockFace.SOUTH;
                        }
                        final BlockState state = block.getState();
                        ((Stairs) (state.getData())).setFacingDirection(face);
                        state.update();
                    }
                }
            }
            width += levelWidth;
        }
    }

    private void placePyramidLayerStep(final Block centerTop, final int yBottom, final BlockModifier mod, final int levelWidth) {
        // TODO: This is a somewhat inefficient implementation.
        final int x0 = centerTop.getX();
        final int y0 = centerTop.getY() - 1;
        final int z0 = centerTop.getZ();
        final World world = centerTop.getWorld();
        int width = 2 * levelWidth;
        for (int y = y0; y >= yBottom; y --) { // Is absolute.
            final int inner = width - 2 * levelWidth; // Coordinate to exempt inside of.
            final int outer = width - levelWidth + 1; // Coordinate to exempt inside of. 
            for (int x = -width; x <= width; x++) { // Relative.
                for (int z = -width; z <= width; z ++) { // Relative.
                    if ((Math.abs(x) > inner || Math.abs(z) > inner) && Math.abs(x) < outer && Math.abs(z) < outer) {
                        mod.apply(world.getBlockAt(x + x0, y, z + z0));
                    }
                }
            }
            width += 2 * levelWidth;
        }
    }

    @SuppressWarnings("deprecation")
    private void placePyramidLayerSnow(final Block centerTop, final int yBottom, final Material material, final int levelWidth) {
        // TODO: This is a somewhat inefficient implementation.
        final int x0 = centerTop.getX();
        final int y0 = centerTop.getY() - 1;
        final int z0 = centerTop.getZ();
        final World world = centerTop.getWorld();
        int width = 8 * levelWidth;
        for (int y = y0; y >= yBottom; y --) { // Is absolute.
            final int inner = width - 8 * levelWidth; // Coordinate to exempt inside of.
            final int outer = width - levelWidth + 1; // Coordinate to exempt inside of. 
            for (int x = -width; x <= width; x++) { // Relative.
                for (int z = -width; z <= width; z ++) { // Relative.
                    if ((Math.abs(x) > inner || Math.abs(z) > inner) && Math.abs(x) < outer && Math.abs(z) < outer) {
                        final Block block = world.getBlockAt(x + x0, y, z + z0);
                        block.setTypeIdAndData(material.getId(), (byte) (7 - Math.max((Math.abs(x) - inner) / levelWidth, (Math.abs(z) - inner) / levelWidth)), true);
                    }
                }
            }
            width += 8 * levelWidth;
        }
    }

}
