package me.asofold.bpl.morecommands.command.editworld;

import me.asofold.bpl.morecommands.MoreCommands;
import me.asofold.bpl.morecommands.command.AbstractCommand;
import me.asofold.bpl.morecommands.command.editworld.EditUtil.BlockSpec;

import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

/**
 * Create Flat ground (erase anything above, set floor, if desired also make blocks below solid).
 * @author asofold
 *
 */
public class SetRadiusCommand extends AbstractCommand<MoreCommands>{

    public SetRadiusCommand(MoreCommands access) {
        super(access, "setradius", "morecommands.command.setradius");
        this.usage = "Specify material[:data] and xz-radius [and one or two y-margins applied from the block you are above].";
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String alias, String[] args) {
        if (!demandPlayer(sender)) {
            return true;
        }

        if (args.length > 5 || args.length < 3) {
            return false;
        }

        // TODO: Allow data value ?
        BlockSpec mod = BlockSpec.matchBlockSpec(args[1]);
        if (mod == null) {
            return false;
        }

        Block refBlock = ((Player) sender).getLocation().getBlock().getRelative(BlockFace.DOWN);

        int xzIndex = 2;
        int y1Index = 3;
        int y2Index = 4;

        Integer xzRadius = null;
        try {
            xzRadius = Integer.parseInt(args[xzIndex]);
            if (xzRadius < 0 || xzRadius > 360) {
                xzRadius = null;
            }
        } catch (NumberFormatException e) {}
        if (xzRadius == null) {
            sender.sendMessage("Unsupported xz-radius: " + args[xzIndex]);
            return false;
        }

        Integer dy1 = null;
        if (args.length > y1Index) {
            try {
                dy1 = Integer.parseInt(args[y1Index]);
                if (dy1 < -256 || dy1 > refBlock.getWorld().getMaxHeight()) {
                    dy1 = null;
                }
            } catch (NumberFormatException e) {}
            if (dy1 == null) {
                sender.sendMessage("Unsupported y-margin: " + args[y1Index]);
                return false;
            }
        } else {
            dy1 = 0;
        }

        Integer dy2 = null;
        if (args.length > y2Index) {
            try {
                dy2 = Integer.parseInt(args[y2Index]);
                if (dy2 < -256 || dy2 > refBlock.getWorld().getMaxHeight()) {
                    dy2 = null;
                }
            } catch (NumberFormatException e) {}
            if (dy2 == null) {
                sender.sendMessage("Unsupported y-margin: " + args[y2Index]);
                return false;
            }
        } else {
            dy2 = 0;
        }

        EditUtil.setRadius(mod, refBlock, xzRadius.intValue(), Math.min(dy1.intValue(), dy2.intValue()), Math.max(dy1.intValue(), dy2.intValue()));

        return true;
    }

}
