package me.asofold.bpl.movelogger;

import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerMoveEvent;
import org.bukkit.event.player.PlayerTeleportEvent;
import org.bukkit.plugin.java.JavaPlugin;

/**
 * Simply log all moving to console for error reproduction with other (more complex) plugins removed.
 * @author asofold
 *
 */
public class MoveLogger extends JavaPlugin implements Listener {

    final Location useLoc = new Location(null, 0, 0, 0);

    @Override
    public void onDisable() {
        getLogger().info(getDescription().getFullName() + " is disabled.");
    }

    @Override
    public void onEnable() {
        getServer().getPluginManager().registerEvents(this, this);
        getLogger().info(getDescription().getFullName() + " is enabled.");
    }

    @EventHandler(priority = EventPriority.LOWEST, ignoreCancelled = true)
    public void onPlayerMove(PlayerMoveEvent event) {
        log(event, null);
    }

    @EventHandler(priority = EventPriority.LOWEST, ignoreCancelled = true)
    public void onPlayerTeleport(PlayerTeleportEvent event) {
        log(event, event.getCause().toString());
    }

    private void log(PlayerMoveEvent event, String extra) {
        Location from = event.getFrom();
        Location to = event.getTo();
        Player player = event.getPlayer();
        Location loc = player.getLocation(useLoc);
        getLogger().info(player.getName() + (loc.equals(from) ? "" : (" loc=" + loc)) + " " + event.getClass().getName() + (extra == null ? "" : (" " + extra)) + " from=" + from + " to=" + to);
        useLoc.setWorld(null);
    }

}
