package me.asofold.bpl.sce;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import me.asofold.bpl.sce.command.SCECommand;

import org.bukkit.Bukkit;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerKickEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.plugin.java.JavaPlugin;

/**
 * Simple ingame yaml editor.<br>
 * Rough order with the simpleconfigeditor(sce) command:
 * <li>options</li>
 * <li>load</li>
 * <li>get/mem+memadd/set</li>
 * <li>save</li>
 * @author asofold
 *
 */
public class SimpleConfigEditor extends JavaPlugin {

    private final Map<UUID, Session> sessionMap = new HashMap<UUID, Session>();

    /** Get ids for non players by name. */
    private final Map<String, UUID> nonPlayerIds = new HashMap<String, UUID>();

    @Override
    public void onEnable() {
        getCommand("simpleconfigeditor").setExecutor(new SCECommand(this));
        Bukkit.getPluginManager().registerEvents(new Listener() {
            @EventHandler(priority = EventPriority.MONITOR, ignoreCancelled = true)
            public void onQuit(PlayerQuitEvent event) {
                onLeave(event.getPlayer().getUniqueId());
            }
            @EventHandler(priority = EventPriority.MONITOR, ignoreCancelled = true)
            public void onQuit(PlayerKickEvent event) {
                onLeave(event.getPlayer().getUniqueId());
            }
            private void onLeave(UUID id) {
                clearSession(id);
            }
        }, this);
        getLogger().info(getDescription().getFullName() + " is enabled.");
    }

    public Session getSession(UUID id) {
        return sessionMap.get(id);
    }

    public Session getOrOpenSession(UUID id) {
        Session session = sessionMap.get(id);
        if (session == null)  {
            session = new Session();
            sessionMap.put(id,  session);
        }
        return session;
    }

    public void clearSession(UUID id) {
        sessionMap.remove(id);
    }

    /**
     * Get the UUID for a player or other command senders. For non-players an id
     * is created by time + name.
     * 
     * @param sender
     * @return
     */
    public UUID getId(CommandSender sender) {
        if (sender instanceof Player) {
            return ((Player) sender).getUniqueId();
        } else {
            UUID id = nonPlayerIds.get(sender.getName());
            if (id == null) {
                id = new UUID(sender.getName().hashCode(), System.nanoTime());
                nonPlayerIds.put(sender.getName(), id);
            }
            return id;
        }
    }

}
