package me.asofold.bpl.sce;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.MemoryConfiguration;

public class Util {

    /**
     * Join parts with link. From NCP.
     * 
     * @param input
     * @param link
     * @return
     */
    public static <O extends Object> String join(final Collection<O> input, final String link)
    {
        final StringBuilder builder = new StringBuilder(Math.max(300, input.size() * 10));
        boolean first = true;
        for (final Object obj : input) {
            if (!first) builder.append(link);
            builder.append(obj.toString());
            first = false;
        }
        return builder.toString();
    }

    public static <O extends Object> String join(O[] input, String link) {
        return join(Arrays.asList(input), link);
    }

    public static <O extends Object> String join(O[] input, int startIndex, String link) {
        return join(Arrays.copyOfRange(input, startIndex, input.length), link);
    }

    /**
     * Copy all values from source to target.
     * @param source
     * @param target
     * @return Number of entries copied.
     */
    public static int copyConfig(MemoryConfiguration source, MemoryConfiguration target) {
        int n = 0;
        for (Entry<String, Object> entry : source.getValues(true).entrySet()) {
            target.set(entry.getKey(), entry.getValue());
            n++;
        }
        return n;
    }

    public static List<String> tabCompleteConfigPath(final MemoryConfiguration config, final String input) {
        // TODO: options + separator char.
        final Set<String> keys = config.getValues(true).keySet();
        final Object value = config.get(input);
        // Special case: input points at a ConfigurationSection entry.
        if (value instanceof ConfigurationSection) {
            // Tab-complete all keys in-order.
            final List<String> output = new LinkedList<String>();
            final String prefix;
            if (input.isEmpty()) {
                prefix = "";
            }
            else {
                prefix = input.charAt(input.length() - 1) == '.' ? input : input + ".";
                output.add(input);
            }
            for (String key : ((ConfigurationSection) value).getKeys(false)) {
                output.add(prefix + key);
            }
            return output;
        }
        final Set<String> matches = new LinkedHashSet<String>();
        // Check if any existing key starts with the input (tab complete minimally, somehow).
        final int length = input.length();
        for (final String key : keys) {
            if (key.startsWith(input)) {
                if (input.equals(key)) {
                    matches.add(key);
                }
                else {
                    // Check if the candidate can be minimized.
                    int index = length + 1;
                    while (key.indexOf('.', index) != -1) { 
                        index = key.indexOf('.', index); // twice though.
                        String ref = key.substring(0, index);
                        if (config.get(ref) != null) {
                            matches.add(ref);
                        }
                        index = Math.max(ref.length(), index + 1);
                    }
                }
            }
        }
        // TODO: Sort ?
        return new ArrayList<String>(matches);
    }

    public static boolean equalsAny(final String input, final String[] reference, final boolean ignoreCase) {
        for (int i = 0; i < reference.length; i++) {
            if (ignoreCase) {
                if (input.equalsIgnoreCase(reference[i])) {
                    return true;
                }
            }
            else if (input.equals(reference[i])) {
                return true;
            }
        }
        return false;
    }

    public static List<String> tabCompleteFirst(final String input, final String[][] variantArrays, final boolean ignoreCase) {
        final String ref = ignoreCase ? input.toLowerCase() : input;
        final List<String> output = new LinkedList<String>();
        for (int i = 0; i < variantArrays.length; i++) {
            String[] variant = variantArrays[i];
            for (int k = 0; k < variant.length; k++) {
                if ((ignoreCase ? variant[k].toLowerCase() : variant[k]).startsWith(ref)) {
                    output.add(variant[k]);
                    break;
                }
            }
        }
        return output;
    }

    public static List<String> tabCompleteSubPath(final File base, final String input) {
        final List<String> output = new LinkedList<String>();
        File ref = new File(base, input);
        if (ref.exists() && ref.isDirectory()) {
            // List all directories and yml files.
            output.add(input);
            addRelativePaths(output, ref, input, true, ".yml");
            return output;
        }
        // Try to split off last part and complete input by entries from the best match directory.
        String parentInput = input.substring(0, input.length() - ref.getName().length());
        ref = new File(base, parentInput);
        if (ref.exists() && ref.isDirectory()) {
            // List all directories and yml files.
            final List<String> refOutput = new LinkedList<String>();
            addRelativePaths(refOutput, ref, parentInput, true, ".yml");
            String prefix = input.substring(parentInput.length());
            for (String path: refOutput) {
                ref = new File(path);
                if (ref.getName().startsWith(prefix)) {
                    output.add(ref.getPath());
                }
            }
        }
        return output;
    }

    /**
     * Add relative paths from directory base on (one level), add pathPrefix + name to output.
     * @param output
     * @param base
     * @param pathPrefix Assume to lead from "the desired start point" to base.
     * @param addDirectories
     * @param fileSuffixes Checked with endsWith, do include '.' for extensions.
     */
    public static void addRelativePaths(final List<String> output, final File base, final String pathPrefix, boolean addDirectories, String ... fileSuffixes) {
        for (String name : base.list()) {
            File file = new File(base, name);
            if (addDirectories && file.isDirectory()) {
                output.add(new File(pathPrefix, name).getPath());
            } else if (file.isFile()) {
                if (fileSuffixes.length > 0) {
                    boolean match = false;
                    for (String ext : fileSuffixes) {
                        if (name.endsWith(ext)) {
                            match = true;
                            break;
                        }
                    }
                    if (!match) {
                        return;
                    }
                }
                output.add(new File(pathPrefix, name).getPath());
            }
        }
    }

}
