package me.asofold.bpl.sce.command;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

import me.asofold.bpl.sce.Session;
import me.asofold.bpl.sce.SimpleConfigEditor;
import me.asofold.bpl.sce.Util;

import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.MemoryConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;

public class LoadCommand extends AbstractCommand<SimpleConfigEditor>{

    public LoadCommand(SimpleConfigEditor access) {
        super(access, "load", null);
    }

    @Override
    public List<String> onTabComplete(CommandSender sender, Command command, String alias, String[] args) {
        if (args.length < 2) {
            return new LinkedList<String>();
        }
        String path = Util.join(args, 1, " ");
        File base = new File(Bukkit.getServer().getWorldContainer(), "plugins");
        return Util.tabCompleteSubPath(base, path);
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String alias, String[] args) {
        // TODO: Consider loading asynchronously.
        // TODO: Restrict to sub path?
        String path = Util.join(args, 1, " ");
        File base = new File(Bukkit.getServer().getWorldContainer(), "plugins");
        File file = new File(base, path);
        if (!file.exists()) {
            sender.sendMessage("Does not exist: " + path);
            return true;
        }
        if (!file.isFile()) {
            sender.sendMessage("Not a file: " + path);
            return true;
        }
        // Load config.
        // TODO: options...
        YamlConfiguration config = new YamlConfiguration();
        try {
            config.load(file);
        } catch (FileNotFoundException e) {
            sender.sendMessage("File not found: " + path);
            return true;
        } catch (IOException e) {
            sender.sendMessage("Could not read: " + path);
            return true;
        } catch (InvalidConfigurationException e) {
            sender.sendMessage("Bad YAML content (see console/log): " + path);
            e.printStackTrace();
            return true;
        }
        // Store session.
        UUID id = access.getId(sender);
        Session session = access.getOrOpenSession(id);
        session.configPath = path;
        session.configFile = file;
        session.config = new MemoryConfiguration();
        int entries = Util.copyConfig(config, session.config);
        sender.sendMessage("YAML file loaded (" + entries + " entries, lists counted once).");
        return true;
    }

}
