package me.asofold.bpl.sce.command;

import java.util.LinkedList;
import java.util.List;

import me.asofold.bpl.sce.Session;
import me.asofold.bpl.sce.SimpleConfigEditor;
import me.asofold.bpl.sce.Util;

import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;

public class SetCommand  extends AbstractCommand<SimpleConfigEditor> {

    /** 0: str, 1: int, 2: double, 3: bool */
    private static String[][] types = new String[][] {
        {"str", "string", "java.lang.String"},
        {"int", "integer", "java.lang.Integer"},
        {"double", "java.lang.Double"},
        {"bool", "boolean", "java.lang.Boolean"}
    };

    public SetCommand(SimpleConfigEditor access) {
        super(access, "set", null);
        // Synchronize with types above.
        usage = "set PATH TYPE CONTENT | set PATH null . TYPE may be str(ing), bool(ean), int(eger), double";
    }

    @Override
    public List<String> onTabComplete(CommandSender sender, Command command, String alias, String[] args) {
        Session session = access.getSession(access.getId(sender));
        if (session == null) {
            sender.sendMessage("No session open.");
            return new LinkedList<String>();
        }
        if (args.length == 3) {
            // Tab complete shortest form of available types.
            return Util.tabCompleteFirst(args[2], types, true);
        }
        else if (args.length == 2) {
            String input = args.length <= 1 ? "" : args[1];
            return Util.tabCompleteConfigPath(session.config, input);
        }
        else if (args.length == 4 && Util.equalsAny(args[2], types[3], true)) {
            return Util.tabCompleteFirst(args[3], new String[][]{{"true"}, {"false"}}, true);
        }
        else {
            // Might tab-complete boolean.
            return new LinkedList<String>();
        }
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String alias, String[] args) {
        // TODO: Allow paths containing spaces, enclosed by '' or similar.
        // TODO: Special cases path parsing?
        // TODO: More types, note usage.
        if (args.length < 3) {
            return false;
        }
        String path = args[1];
        Session session = access.getSession(access.getId(sender));
        if (session == null) {
            sender.sendMessage("No session open.");
            return true;
        }
        String typeSpec = args[2];
        // Special types.
        if (typeSpec.equalsIgnoreCase("null")) {
            if (args.length != 3) {
                sender.sendMessage("Can not specify a value with null.");
                return false;
            }
            else {
                session.config.set(path, null);
                sender.sendMessage("Set to null.");
                return true;
            }
        }
        // Types with values.
        if (args.length < 4) {
            sender.sendMessage("Must specify a value.");
            return true;
        }
        String value = Util.join(args, 3, " ");
        if (Util.equalsAny(typeSpec, types[0], true)) {
            session.config.set(path, value);
            sender.sendMessage("String value set.");
            return true;
        }
        else if (Util.equalsAny(typeSpec, types[1], true)) {
            Integer intVal = null;
            try {
                intVal = Integer.parseInt(value);
            }
            catch (NumberFormatException e) {
            }
            if (intVal == null) {
                sender.sendMessage("Bad value for Integer: " + value);
                return true;
            }
            else {
                session.config.set(path, intVal);
                sender.sendMessage("Integer value set.");
                return true;
            }
        }
        else if (Util.equalsAny(typeSpec, types[2], true)) {
            Double doubleVal = null;
            try {
                doubleVal = Double.parseDouble(value);
            }
            catch (NumberFormatException e) {
            }
            if (doubleVal == null) {
                sender.sendMessage("Bad value for Double: " + value);
                return true;
            }
            else {
                session.config.set(path, doubleVal);
                sender.sendMessage("Double value set.");
                return true;
            }
        }
        else if (Util.equalsAny(typeSpec, types[3], true)) {
            Boolean booleanVal = null;
            if (value.equalsIgnoreCase("true")) {
                booleanVal = true;
            }
            else if (value.equalsIgnoreCase("false")) {
                booleanVal = false;
            } else {
                sender.sendMessage("Bad value for Boolean: " + value);
                return true;
            }
            session.config.set(path, booleanVal);
            sender.sendMessage("Boolean value set.");
            return true;
        }
        else {
            return false;
        }
    }

}
